require 'yaml'
require 'pathname'
require 'active_record'
require 'locale'

require_relative 'web_connect/model'

module Netfira
  module WebConnect
    ROOT = Pathname(__FILE__).parent

    def self.configure(input = nil, **options)
      if Hash === input
        @config = Configuration.new(input)
      elsif String === input
        @config = Configuration.new(YAML.parse input)
      elsif input.respond_to? :read
        @config = Configuration.new(YAML.parse_stream input)
      elsif input.nil?
        @config ||= Configuration.new
      else
        raise "You cannot configure WebConnect with an instance of #{input.class}"
      end
      @config.materialize_when_db_changed = !options[:skip_materialization]
      yield @config if block_given?
      Paperclip::Attachment.default_options[:preserve_files] = paranoia?
    end

    def self.db_table_prefix(string = '')
      [@config.db_table_prefix, string].join.to_sym
    end

    def self.db_table_l10n_suffix(table_name = '')
      :"#{table_name}_l10n"
    end

    def self.schema_migrations_table_name
      db_table_prefix('schema_migrations').to_s
    end

    def self.logger
      @config.logger || null_logger
    end

    def self.system_locale
      @system_locale ||= Locale.current.to_simple.to_s
    end

    def self.file_store
      @config.file_store || {}
    end

    def self.authenticator
      @config.authenticator
    end

    def self.anonymous_shop
      @anonymous_shop ||= Models::Shop.find(0)
    end

    def self.schema
      @schema ||= Schema.new if @config
    end

    def self.session_lifetime
      @config.session_lifetime
    end

    def self.paranoia?
      !!@config.paranoia
    end

    def self.http_notifications
      @config.http_notifications || :none
    end

    def self.amqp_uri
      @config.amqp
    end

    private

    def self.null_logger
      @null_logger ||= Logger.new nil
    end

  end
end

require_relative 'web_connect/events'

require_relative 'web_connect/rack_app'
require_relative 'web_connect/request_filter'
require_relative 'web_connect/migration'
require_relative 'web_connect/version'
require_relative 'web_connect/models'
require_relative 'web_connect/components'
require_relative 'web_connect/configuration'
require_relative 'web_connect/schema'
require_relative 'web_connect/amqp'
