require 'bunny'

module Netfira::WebConnect
  module AMQP; end
  class << AMQP

    def connect!
      disconnect!
      @connection = Bunny.new(Netfira::WebConnect.amqp_uri)
      @connection.start
      @channel = @connection.create_channel
      @exchanges = {}
    end

    def connected?
      !!@connection
    end

    def disconnect!
      return unless connected?
      @connection.stop
      @connection = nil
      @channel = nil
      @exchanges = nil
    end

    def exchange_for_shop(shop)
      @exchanges && @exchanges[shop.id || 0] ||= make_exchange_for_shop(shop)
    end

    def queue_name_for_shop(shop)
      "shops.#{shop.id || 0}"
    end

    private

    def make_exchange_for_shop(shop)

      # Each shop will have its own exchange, and a queue bound to that exchange.
      # Their names will be the same.
      name = queue_name_for_shop(shop)

      # Make an exchange
      @connection.direct(name).tap do |exchange|

        # Make a queue and bind it to the created exchange
        @channel.queue(name, auto_delete: true).bind exchange, routing_key: name
      end
    end

  end
end
