module Netfira::WebConnect
  class HashOfModels < Hash

    def initialize
      super { |hash, key| hash[key] = [] }
    end

    def [](key)
      super class_for_key(key)
    end

    def <<(value)
      raise 'Only models are acceptable' unless Model === value
      self[value.class] << value
    end

    def method_missing(sym, *args, &block)
      if respond_to_missing? sym
        klass = class_for_key(sym)
        -> { self[klass] }.tap { |method| self.class.__send__ :define_method, sym, method }.call
      else
        super sym, *args, &block
      end
    end

    def respond_to_missing?(sym)
      sym !~ /[^\w]/ && !!class_for_key(sym)
    end

    def as_json(options = nil)
      map do |klass, records|
        is_relation = klass < Model::Relation
        [
            klass.name.demodulize.to_sym,
            records.map { |record| is_relation ? record.records.map(&:origin_id) : record.origin_id }
        ]
      end.to_h
    end

    private

    def class_for_key(key)
      return key if Class === key and key < Model
      class_name = key.to_s.demodulize.camelize.singularize.to_sym
      if Models.const_defined? class_name
        klass = Models.const_get(class_name)
        klass < Model and klass
      end
    end

  end
end
