require 'active_support/inflector'

class Netfira::WebConnect::OctaWord

  class << self

    def from_binary(digest)
      new digest if digest
    end

    alias_method :from, :from_binary

    def from_base64(digest)
      new (digest[0..21] << '==').unpack('m').first
    end

    def from_hex(digest)
      new [digest].pack('H*')
    end

  end

  def initialize(binary_digest)
    @binary = binary_digest.b
  end

  def binary
    @binary
  end

  alias_method :b, :binary

  def hex
    @binary.unpack('H*')[0]
  end

  def base64
    [@binary].pack('m0')[0..21]
  end

  def to_s
    base64
  end

  def inspect
    "#{self.class.name.demodulize}: #{base64}"
  end

  def eql?(other)
    case other
      when Netfira::WebConnect::OctaWord then b == other.b
      when String
        case other.length
          when 16 then other == b
          when 22 then other == base64
          when 32 then other == hex
          else false
        end
      else false
    end
  end

  alias_method :==, :eql?

  def as_json(options = nil)
    base64
  end

end

# Subclasses:
require_relative 'guid'
require_relative 'checksum'
