module Netfira::WebConnect
  class Configuration

    attr_accessor :db_table_prefix, :file_store, :authenticator, :custom_fields,
                  :time_zone, :materialize_when_db_changed, :session_lifetime, :paranoia

    attr_reader :logger, :db, :http_notifications, :amqp

    def initialize(defaults = nil)
      defaults.each{ |k, v| __send__ :"#{k}=", v } if defaults
    end

    def on_authenticate(proc = nil, &block)
      authenticator = proc || block
      raise 'Authenticator must take exactly two arguments (shop ID and password)' unless authenticator.arity == 2
      self.authenticator = authenticator
    end

    def db=(details)
      @db = details
      if details
        Model.establish_connection details
        Models.materialize if materialize_when_db_changed && !Netfira::WebConnect.needs_migration?
      end
    end

    def logger=(logger)
      @logger = logger
      Model.logger = logger if logger
    end

    def http_notifications=(value)
      allowed = %i[none sync async]
      raise "Valid values for `http_notifications` are #{allowed.map { |x| ":#{x}" }.join ', '}" unless allowed.include? value
      @http_notifications = value
    end

    def amqp=(value)
      return if value == @amqp
      allowed = %w[amqp amqps]
      uri = URI(value)
      raise "Scheme must be one of [ #{allowed.join ' | '} ]" unless allowed.include? uri.scheme
      @amqp = value
      AMQP.connect!
    end

  end
end
