module Netfira::WebConnect

  def self.event_delegates
    @event_delegates ||= []
  end

  def self.dispatch_event(timing, event, *args)

    # Allow multiple timings to be dispatched simultaneously, e.g. [:on, :after]
    return timing.each{ |t| dispatch_event(t, event, *args) { yield } } if timing.respond_to? :each

    # The method to invoke, e.g. :on_update_product
    target = :"#{timing}_#{event}"

    # Delegates that respond to this method
    delegates = event_delegates.select { |d| d.respond_to? target }

    # 'Around' events get special treatment
    if timing == :around
      chain_runner = -> do
        delegate = delegates.shift
        if delegate
          delegate.__send__ target, *args, &chain_runner
        else
          yield
        end
      end
      chain_runner.call
    else
      delegates.map { |d| d.__send__ target, *args }
    end
  end

  def self.handles_event?(event)
    target = :"on_#{event}"
    !!event_delegates.find { |x| x.respond_to? target }
  end

end
