require 'net/https'

module Netfira::WebConnect
  class Model::Record

    module Sendable
      extend ActiveSupport::Concern

      DELIVERY_STATUSES = %i(unsent sent delivered)

      included do

        # Make #delivery_status an enumeration, generating methods like #sent?, #sent!, etc.
        enum delivery_status: DELIVERY_STATUSES

        # Scopes
        sent_and_undelivered_index = DELIVERY_STATUSES.index(:sent)
        scope :sent_and_undelivered, -> { where delivery_status: sent_and_undelivered_index }

      end

      def send!
        sent!
        save!
        send_notifications!
        self
      end

      private

      def send_notifications!
        send_http_notification!
        send_amqp_notification!
      end

      def send_http_notification!
        case Netfira::WebConnect.http_notifications
          when :sync then send_http_notification_now!
          when :async then Thread.new { send_http_notification_now! }
          else return
        end
      end

      def send_http_notification_now!
        uri_string = shop.settings['notify']
        if uri_string && !uri_string.empty?
          uri = URI(uri_string)
          Net::HTTP.get_response uri if %w[http https].include? uri.scheme
        end
      end

      AMQP_MESSAGE_TTL = 60 * 1000 # Messages should live for 60 seconds

      def send_amqp_notification!
        shop.amqp_publish '', type: 'fetch', expiration: AMQP_MESSAGE_TTL
      end

    end

  end
end
