require 'bigdecimal'

module Netfira::WebConnect
  class Model::Record

    module Serializer
      extend ActiveSupport::Concern

      EXCLUDE_FROM_SERIALIZE = %w[id shop_id digest delivery_status created_at updated_at deleted_at]

      included do

        private

        # Returns an array of three arrays of keys, e.g.
        # [ ['product_id', 'unit_price'], ['description'], [] ]
        def self.attributes_to_serialize
          @attributes_to_serialize ||= [
              attribute_names - EXCLUDE_FROM_SERIALIZE,
              has_languages? ? self::Translation.translated_attribute_names : [],
              [] # Custom attributes (not implemented)
          ]
        end
      end

      def to_s

        # Map each array of keys from the above method into a sorted hash of key/value pairs
        data = self.class.attributes_to_serialize.map do |keys|
          pairs = keys.sort.map do |key|
            [key.camelize(:lower), cast_for_serialization(__send__ key)]
          end
          pairs.reject!{ |v| v[1].nil? }.to_h
        end

        # Return the class name and the sets above, e.g. Product{...},{...},{...}
        self.class.name.demodulize << JSON.generate(data)[1..-2]
      end

      private

      def cast_for_serialization(value)
        if TranslatedString === value
          value = value.sort
          value = value.empty? ? nil : value.to_h
        elsif Numeric === value
          value = (value == value.floor) ? value.to_i : value.to_f
        elsif OctaWord === value
          value = value.to_s
        end
        value
      end

    end
  end
end
