require 'forwardable'

module Netfira::WebConnect

  # This class represents a translated string on a model, e.g. product.description.
  # Callbacks are supplied by the Model::Record::Translations mixin
  class Model::Record::TranslatedString
    extend Forwardable

    def initialize
      @callbacks = {}
    end

    def [](lang)
      @callbacks[:get].call lang
    end

    def []=(lang, value)
      @callbacks[:set].call lang, value
    end

    def on(action, &block)
      @callbacks[action] = block
    end

    def to_s
      self[nil]
    end

    def to_h
      @callbacks[:all].call
    end

    def merge!(values)
      values.each{ |key, value| self[key] = value }
      self
    end

    delegate %i[count length size each map sort select reject] => :to_h

  end
end
