require 'ostruct'

module Netfira::WebConnect
  class Models::Setting < Model::Support

    belongs_to :shop, inverse_of: :setting_models

    around_save :dispatch_change_event
    before_destroy :dispatch_destroy_event

    def typed_value
      cast_by_content_type value, content_type
    end

    def typed_value=(value)
      if String === value
        if value.encoding.name == 'ASCII-8BIT'
          self.value = value.b
          self.content_type = 'application/octet-stream'
        else
          self.value = value.force_encoding('UTF-8')
          self.content_type = 'text/plain;charset=UTF-8'
        end
      else
        self.value = JSON.generate(value, quirks_mode: true)
        self.content_type = 'application/json'
      end
    end

    private

    def cast_by_content_type(value, content_type)
      case content_type
        when 'application/json' then JSON.parse(value, quirks_mode: true)
        when 'application/octet-stream' then value.b
        when /^text\/plain;charset=(.+)$/ then value.force_encoding($1)
        else value
      end
    end

    def dispatch_change_event
      old = persisted? ? cast_by_content_type(changed_attributes['value'], changed_attributes['content_type']) : nil
      yield
      new = OpenStruct.new(shop: shop, key: key, value: typed_value)
      dispatch_event [:on, :after], 'change_setting', new, old unless old == new.value
    end

    def dispatch_destroy_event
      return unless persisted?
      new = OpenStruct.new(shop: shop, key: key, value: nil)
      old = typed_value
      dispatch_event [:before, :on], 'change_setting', new, old unless old.nil?
      true
    end

  end
end
