module Netfira::WebConnect
  class Models::Shop < Model::Support

    has_many :setting_models, class_name: Models::Setting.name, inverse_of: :shop
    has_many :sessions

    def settings
      @settings ||= Settings.new(setting_models)
    end

    def reload
      @settings = nil
      super
    end

    def locale
      settings[:locale] || Netfira::WebConnect.system_locale
    end

    def find_relation_by_origin_ids(**args)
      raise ArgumentError, "Provide exactly 2 origin IDs, e.g. :product => 'abc'" unless args.length == 2
      classes = args.keys.map do |type|
        name = type.to_s.camelize.to_sym
        Models.const_get(name).tap { |k| raise "#{type} isn't a valid record type. Use singular snake-case, e.g. :product" unless k < Model::Record }
      end
      relation_class = Model::Relation.for!(*classes)
      relation_class.with_deleted.find_by_origin_ids(args.merge shop: self).first
    end

    def amqp_exchange
      AMQP.exchange_for_shop self
    end

    def amqp_queue
      AMQP.queue_name_for_shop self
    end

    def amqp_publish(message, **opts)
      amqp_exchange && amqp_exchange.publish(message, **opts.merge(routing_key: amqp_queue))
    end

  end
end

require_relative 'shop/settings'
