require 'forwardable'
require 'active_support/core_ext/object/deep_dup'

module Netfira::WebConnect
  class Models::Shop::Settings
    extend Forwardable

    def initialize(proxy)
      @proxy = proxy
      @models = proxy.map{ |model| [model.key.to_sym, model] }.to_h
      @hash = @models.map{ |key, model| [key, model.typed_value]}.to_h
    end

    def [](key)
      @hash[key.to_sym]
    end

    def []=(key, value)
      key = key.to_sym
      model = @models[key] ||= @proxy.new(key: key.to_s)
      model.typed_value = value
      model.save!
      @hash[key] = value
    end

    def to_h
      @hash.deep_dup
    end

    def delete(key)
      key = key.to_sym
      return false unless @models[key]
      @models[key].destroy!
      @models.delete key
      @hash.delete key
      true
    end

    delegate [:count, :length, :size, :each, :map] => :@hash

  end
end
