require 'active_support/inflector'
require 'active_support/hash_with_indifferent_access'

#todo needs comment
module Netfira::WebConnect
  class RackApp::Action
    include RackApp::Exceptions::HttpExceptions

    def self.action_classes
      @action_classes ||= find_action_classes
    end

    def self.latest_version
      @latest_version ||= action_classes.keys.max
    end

    def self.create(action, version = nil)
      version ||= latest_version
      klass = nil
      until klass or version < 1
        klass = (action_classes[version] || {})[action] # todo needs explination or to be rewritten to be easily readable
        version -= 1
      end
      klass and klass.new
    end

    def initialize
      @headers = ActiveSupport::HashWithIndifferentAccess.new
    end

    def dispatch_event(*args, &block)
      Netfira::WebConnect.dispatch_event *args, &block
    end

    private

    def self.find_action_classes
      action_classes = {}

      # Loop through all constants in this class's namespace
      constants.each do |constant|

        # Match Version# constants (modules containing actions)
        if constant =~ /\AVersion(\d+)\z/

          # The module containing the actions
          mod = const_get(constant)

          # A hash of actions, e.g. :commit => Action::Version8::Commit
          action_classes[$1.to_i] = Hash[mod.constants.map do |name|
            mod.const_get(name)
          end.select do |klass|
            klass < self
          end.map do |klass|
            [klass.name.demodulize.underscore.to_sym, klass]
          end]
        end
      end

      action_classes
    end

  end
end

# Load action helpers
Dir[File.expand_path('../action_helpers/*.rb', __FILE__)].each{ |f| require f }

# Pre load all action classes in all versions
Dir[File.expand_path('../actions/version_*/*.rb', __FILE__)].each{ |f| require f }
