module Netfira::WebConnect
  class RackApp::Action

    attr_reader :timeout, :input, :path, :verb, :shop, :headers, :query_string, :env

    # This method sets a response header, e.g.header 'Content-Type', 'text/plain'
    def header(name, value)
      name = name.to_s.split(/[- _]/).map(&:capitalize).join('-').to_sym
      if value
        headers[name] = value
      else
        headers.delete name
      end
    end

    # These methods tell you if a given verb was used, e.g. post?
    %w[get post put delete].each do |verb|
      define_method(:"#{verb}?") { self.verb.to_s == verb }
    end

    # This method restricts request verbs (symbols) and input types (classes)
    def allow(*args)

      # Restrict verbs
      verbs = args.select { |x| Symbol === x }
      unless verbs.empty? || verbs.include?(verb)
        raise MethodNotAllowed.new("The #{verb.to_s.upcase} verb is not allowed", allowed: verbs.map{ |v| v.to_s.upcase })
      end

      # Restrict input type
      types = args.select { |x| Class === x }
      unless types.empty? || types.find { |t| t === input }
        raise BadRequest.new('Unexpected request body type', allowed: types.map { |t| t.name })
      end
    end

  end
end
