module Netfira::WebConnect
  module RackApp::Action::Version8
    class Commit < RackApp::Action

      private

      def commit_records(records)
        records.each do |class_name, attribute_sets|
          klass = class_for_record_type(class_name)
          attribute_sets.each do |attributes|
            modify_record klass, attributes
          end
        end
      end

      def modify_record(klass, attributes)
        record = find_record(klass, attributes)
        if attributes['@delete']
          delete_record record
        else
          update_record record, attributes
        end
        complete_record record
      end

      def update_record(record, attributes)
        klass = record.class
        origin_key = klass.origin_key.to_s
        attributes.each do |key, value|
          key = key.underscore
          next if key == origin_key
          if key == 'checksum' && klass < Model::Record::FileRecord && !attributes['remoteLocation']
            request_file klass, attributes['fileName'] if record.checksum != value
            next
          end
          record.write_with_type_casting(key.to_sym, value) or unknown_attribute! klass, key
        end
        record.save or could_not_save! record
      end

      def delete_record(record)
        record.destroy! if record.persisted?
      end

      def complete_record(record)
        completed(:records) << record
        list = complete[:records][record.class.name.demodulize] ||= {}
        list[record.origin_id.to_s] = record.digest
      end

      def request_file(klass, file_name)
        list = files_to_send[klass.name.demodulize] ||= []
        list << file_name unless list.include? file_name
      end

      def find_record(klass, attributes)
        origin_key = klass.origin_key.to_s.camelize :lower
        origin_id = attributes[origin_key]
        raise BadRequest, "#{klass.name.demodulize} sent with no #{klass.origin_key}" unless origin_id
        klass.find_or_initialize_by_origin_id(shop, origin_id.to_s)
      end

      def could_not_save!(record)
        message = "#{record.class.name.demodulize} record could not be saved"
        errors = record.errors.messages.map { |k, v| [k.to_s.camelize(:lower), v] }.to_h
        raise BadRequest.new message, errors: errors
      end

      def unknown_attribute!(klass, key)
        raise BadRequest, "#{klass.name.demodulize.pluralize} don't have #{key} fields"
      end

    end
  end
end
