class Netfira::WebConnect::RackApp
  module Action::Version8
    class Settings < Action
      def call
        allow :put, :post, :get, :delete
        key = path && path.first
        if key
          handle_single key
        else
          handle_multiple
        end
      end

      private

      def handle_single(key)
        case verb
          when :put, :post then update_single key
          when :get then get_single key
          when :delete then delete_single key
          else raise MethodNotAllowed
        end
      end

      def update_single(key)
        if input.respond_to? :read
          setting = shop.setting_models.find_or_initialize_by(key: key)
          setting.value = input.read
          setting.content_type = env['CONTENT_TYPE'] || 'application/x-octet-stream'
          setting.save
        else
          shop.settings[key] = input
        end
        {}
      end

      def get_single(key)
        setting = shop.setting_models.find_by(key: key)
        raise NotFound unless setting
        header 'Content-Type', setting.content_type
        setting.value
      end

      def delete_single(key)
        shop.setting_models.where(key: key).destroy_all
        {}
      end

      def handle_multiple
        case verb
          when :put, :post then update_multiple
          when :get then get_all
          when :delete then delete_all
          else raise MethodNotAllowed
        end
      end

      def update_multiple
        allow Hash
        input.each do |key, value|
          shop.settings[key] = value.unpack('m').first
        end
        {}
      end

      def get_all
        {settings: shop.setting_models.map{ |model| [model.key, [model.value.b].pack('m0')] }.to_h}
      end

      def delete_all
        shop.setting_models.destroy_all
        {}
      end

    end
  end
end
