require 'rack/utils'
require 'forwardable'

module Netfira::WebConnect::RackApp::Exceptions

  class HttpException < Base
    extend Forwardable

    class << self
      attr_accessor :status, :code, :category
      attr_reader :headers
    end

    def_delegators :'self.class', :status, :code, :category

    def initialize(message = nil, details = nil)
      super message || self.class.name.demodulize.underscore.humanize
      @headers = self.class.headers.dup
      @details = details
    end

    def body
      @body ||= make_body
    end

    attr_reader :headers

    private

    def make_body
      body = {errorMessage: message}
      body.merge! @details if Hash === @details
      body
    end

  end

  module HttpExceptions

    Rack::Utils::HTTP_STATUS_CODES.each do |code, status|
      next if status == 'Unassigned'
      const_set status.gsub(/[ -](\w)/){$1.upcase}.sub(/ .*/, ''), (Class.new(HttpException) do

        @status = status
        @code = code
        @category = code / 100

        @headers = {}

      end)
    end

    Unauthorized.headers['WWW-Authenticate'] = 'Basic realm="Authentication required"'

  end

end
