module Netfira::WebConnect

  class Schema::Table

    attr_reader :klass

    def initialize(klass)
      @klass = klass
    end

    def singular
      @singular ||= klass.name.demodulize
    end

    def plural
      @plural ||= singular.pluralize
    end

    def primary_key
      @primary_key ||= klass.origin_key
    end

    def localize
      @localize ||= l10n_columns.keys.map(&:to_sym)
    end

    def relations
      @relations = klass.related_classes
    end

    def has_many
      @has_many ||= klass.reflect_on_all_associations(:has_many).select do |a|
        a.klass < Model::Record && !a.options[:through]
      end.map do |association|
        [association.name, association.klass]
      end.to_h
    end

    IGNORE_COLUMNS = %w[id shop_id created_at updated_at deleted_at digest]

    def columns
      @columns ||= klass.columns_hash.reject{ |k| IGNORE_COLUMNS.include? k }.merge(l10n_columns).map{ |k, v| [k.to_sym, v.type.to_s] }.to_h
    end

    def as_json(options)
      @json ||= {
          columns:    columns.map { |name, type| [name.to_s.camelize(:lower), type] }.to_h,
          relations:  relations.map{ |klass| klass.name.demodulize.pluralize },
          singular:   singular,
          primaryKey: [primary_key.to_s.camelize(:lower)],
          localize:   l10n_columns.keys.map{ |k| k.camelize :lower },
          hasMany:   has_many.map{ |type, klass| [type, klass.plural_name.camelize] }.to_h
      }
    end

    private

    def l10n_columns
      @l10n_columns ||= klass.has_languages? ? l10n_filtered_columns_hash : {}
    end

    IGNORE_L10N_COLUMNS = %w[id language]

    def l10n_filtered_columns_hash
      klass::Translation.columns_hash.reject do |k|
        k == "#{klass.single_name}_id" or IGNORE_L10N_COLUMNS.include? k
      end
    end

  end

end
